﻿using CMS.PortalEngine;
using CMS.Base;
using CMS.CustomTables;
using CMS.Helpers;
using System;
using CMS.OutputFilter;
using System.Text;
using CMS.EventLog;
using CMS.DataEngine;
using Custom;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using CMS.MacroEngine;

[ContentReplacementTableLoader]
public partial class CMSModuleLoader
{
    public class ContentReplacementTableLoader : CMSLoaderAttribute
    {
        /// <summary>
        /// Initializes the module
        /// </summary>
        public override void Init()
        {
            // Ensures that the an output filter instance is created on every request
            RequestEvents.PostMapRequestHandler.Execute += PostMapRequestHandler_Execute;
        }

        private void PostMapRequestHandler_Execute(object sender, EventArgs e)
        {
            // Creates an output filter instance
            ResponseOutputFilter.EnsureOutputFilter();

            // Assigns a handler to the OutputFilterContext.CurrentFilter.OnAfterFiltering event
            OutputFilterContext.CurrentFilter.OnAfterFiltering += CurrentFilter_OnAfterFiltering;
        }


        void CurrentFilter_OnAfterFiltering(ResponseOutputFilter filter, ref string finalHtml)
        {
            try
            {
                // Replace static relative URLs with absolute cookieless URLs
                // Only do this for the Live site
                if (PortalContext.ViewMode == ViewModeEnum.LiveSite)
                {
                    // Get the transformation text for the replacement content template
                    TransformationInfo tinfo = TransformationInfoProvider.GetTransformation("custom.contenturlreplacementtable.replacementurl");

                    // Try to get the data from cache
                    var items = CacheHelper.Cache(() => CustomTableItemProvider.GetItems("Custom.ContentURLReplacementTable").TypedResult,
                      // Cache for 60 minutes with "contenturlreplacementvalues" cache key
                      new CacheSettings(60, "contenturlreplacementvalues")
                      {
                          // Setup the cache dependencies, flush the cache if any of the content replacement values are changed
                          CacheDependency = CacheHelper.GetCacheDependency("customtableitem.custom.contenturlreplacementtable|all")
                      }
                    );

                    foreach (var item in items)
                    {
                        // Make sure the item has am OriginalContent value
                        if (item.GetStringValue("OriginalContent", "") != "")
                        {
                            string strOriginalContent = " " + item.GetStringValue("OriginalContent", "") + " ";

                            // Check if the output contains the value
                            if (finalHtml.IndexOf(strOriginalContent) > -1)
                            {
                                // Create a new instnace of the resolver
                                var resolver = MacroResolver.GetInstance();

                                // Set the NamedSourceData to the item. We will reference the properties in the transformation by Item.[Property]
                                resolver.SetNamedSourceData("Item", item);

                                // Replace the content
                                finalHtml = finalHtml.Replace(strOriginalContent, " " + resolver.ResolveMacros(tinfo.TransformationCode) + " ");
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                // Log any errors
                EventLogProvider.LogException("ContentReplacementTableLoader", "CurrentFilter_OnAfterFiltering", ex);
            }
        }
    }
}

